require_relative 'player'
require_relative 'game_turn'
require_relative 'treasure_trove'

module StudioGame
  class Game
    attr_reader :title
  
    def initialize(title)
      @title = title
      @players = []
    end
  
    def load_players(from_file)
      File.readlines(from_file).each do |line|
        add_player(Player.from_csv(line))
      end
    end

    def add_player(player)
      @players << player
    end
  
    # def print_name_and_health(player)
    #   puts "#{player.name} (#{player.health})"
    # end
  
    def high_score_entry(player)
      formatted_name = player.name.ljust(20, '.')
      "#{formatted_name} #{player.score}"
    end
  
    def print_stats
      puts "\n#{title} Statistics:"
      strong, weak = @players.partition { |player| player.strong? }
    
      puts "\n#{strong.size} strong players:"
      strong.each do | player |
        puts player.to_name_and_health
        # print_name_and_health(player)
      end
      
      puts "\n#{weak.size} weak players:"
      weak.each do | player |
        puts player.to_name_and_health
        # print_name_and_health(player)
      end

      puts "\n#{title} High Scores:\n"
      @players.sort.each do |player| 
        puts high_score_entry(player)
      end
    
      @players.each do |player|
        puts "\n#{player.name}'s point totals:"
        player.each_found_treasure do |treasure|
          puts "#{treasure.points} total #{treasure.name} points"
        end
        puts "#{player.points} grand total points"
      end   
    
      puts "\n#{total_points} total points from treasures found"
    end
  
    def total_points
      @players.reduce(0) { | sum, player | sum + player.points }
    end
  
    def save_high_scores(to_file = "high_scores.txt")
      File.open(to_file, "w") do |file|
        file.puts "#{@title} High Scores:"
        @players.each do |player|
          file.puts high_score_entry(player)
        end
      end
    end
    
    def play(rounds)
      puts "\nThere are #{@players.size} players in #{@title}."
    
      treasures = TreasureTrove::TREASURES
      puts "\nThere are #{treasures.size} treasures to be found:"
      treasures.each do |treasure|
        puts "A #{treasure.name} is worth #{treasure.points} points"
      end
    
      1.upto(rounds) do |round|
        if block_given?
          break if yield
        end
        puts "\nRound #{round}:"
      
        # @players.each do |player|
   #        puts player
   #      end
   # 
        @players.each do |player|
          GameTurn.take_turn(player)
          puts player
        end
      end
    end 
  end
end
