require_relative 'treasure_trove'
require_relative 'playable'

module StudioGame
  class Player 
    include Playable
  
    attr_reader :reader
    attr_accessor :name, :health
  
    def initialize(name, health = 100)
      @name = name.capitalize
      @health = health
      @found_treasures = Hash.new(0)
    end
  
    def self.from_csv(line)
      name, health = line.split(',')
      Player.new(name, Integer(health.to_i))
    end
    
    def to_name_and_health
      "#{@name} (#{@health})"
    end
 
    def to_s
      "I'm #{@name } with health = #{@health}, points = #{points}, and score = #{score}."
    end
  
    def score
      @health + points
    end
  
    def name=(new_name)
      @name = new_name.capitalize
    end
  
    def found_treasure(treasure)
      @found_treasures[treasure.name] += treasure.points
      puts "#{@name} found a #{treasure.name} worth #{treasure.points} points."
      puts "#{@name}'s treasures: #{@found_treasures}"
    end
  
    def each_found_treasure
      # I got the following line wrong first time
      # when iterating over a hash you provide 2 vars for key and value
      # and stop putting 'for' in front of iterators!!!
     @found_treasures.each do | name, points |
        yield Treasure.new(name, points)
      end
    end
  
    def points
      @found_treasures.values.reduce(0, :+)
    end
  
    def <=>(other_player)
      other_player.score <=> score
    end 
  end

  if __FILE__ == $0
    player = Player.new("moe")
    puts player.name
    puts player.health
    player.w00t
    puts player.health
    player.blam
    puts player.health
  end
end
